@@ -45,7 +45,7 @@
 #include "cpl_vsi.h"
 #include "gdal.h"
 
-CPL_CVSID("$Id: rasterfill.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id: rasterfill.cpp 9aecbf344bcaf55e1ef26aa89f0732f60a644ff4 2021-08-11 11:10:23 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                           GDALFilterLine()                           */
@@ -56,10 +56,10 @@ CPL_CVSID("$Id: rasterfill.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-
 /************************************************************************/
 
 static void
-GDALFilterLine( float *pafLastLine, float *pafThisLine, float *pafNextLine,
+GDALFilterLine( const float *pafLastLine, const float *pafThisLine, const float *pafNextLine,
                 float *pafOutLine,
-                GByte *pabyLastTMask, GByte *pabyThisTMask, GByte*pabyNextTMask,
-                GByte *pabyThisFMask, int nXSize )
+                const GByte *pabyLastTMask, const GByte *pabyThisTMask, const GByte *pabyNextTMask,
+                const GByte *pabyThisFMask, int nXSize )
 
 {
     for( int iX = 0; iX < nXSize; iX++ )
@@ -804,11 +804,10 @@ GDALFillNodata( GDALRasterBandH hTargetBand,
             {
                 if( adfQuadDist[iQuad] <= dfMaxSearchDist )
                 {
-                    const double dfWeight = 1.0 / adfQuadDist[iQuad];
-
-                    bHasSrcValues = dfWeight != 0;
+                    bHasSrcValues = true;
                     if( !bHasNoData || fQuadValue[iQuad] != fNoData )
                     {
+                        const double dfWeight = 1.0 / adfQuadDist[iQuad];
                         dfWeightSum += dfWeight;
                         dfValueSum += fQuadValue[iQuad] * dfWeight;
                     }
@@ -817,7 +816,6 @@ GDALFillNodata( GDALRasterBandH hTargetBand,
 
             if( bHasSrcValues )
             {
-                pabyMask[iX] = 255;
                 pabyFiltMask[iX] = 255;
                 if( dfWeightSum > 0.0 )
                     pafScanline[iX] = static_cast<float>(dfValueSum / dfWeightSum);
