@@ -36,7 +36,7 @@
 
 #include <limits>
 
-CPL_CVSID("$Id: ersdataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id: ersdataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -89,7 +89,7 @@ class ERSDataset final: public RawDataset
     ERSDataset();
     ~ERSDataset() override;
 
-    void FlushCache(void) override;
+    void FlushCache(bool bAtClosing) override;
     CPLErr GetGeoTransform( double * padfTransform ) override;
     CPLErr SetGeoTransform( double *padfTransform ) override;
     const char *_GetProjectionRef(void) override;
@@ -160,7 +160,7 @@ ERSDataset::ERSDataset() :
 ERSDataset::~ERSDataset()
 
 {
-    ERSDataset::FlushCache();
+    ERSDataset::FlushCache(true);
 
     if( fpImage != nullptr )
     {
@@ -212,7 +212,7 @@ int ERSDataset::CloseDependentDatasets()
 /*                             FlushCache()                             */
 /************************************************************************/
 
-void ERSDataset::FlushCache()
+void ERSDataset::FlushCache(bool bAtClosing)
 
 {
     if( bHDRDirty )
@@ -233,7 +233,7 @@ void ERSDataset::FlushCache()
         }
     }
 
-    RawDataset::FlushCache();
+    RawDataset::FlushCache(bAtClosing);
 }
 
 /************************************************************************/
@@ -840,6 +840,13 @@ int ERSDataset::Identify( GDALOpenInfo * poOpenInfo )
 /************************************************************************/
 
 namespace {
+
+static int& GetRecLevel()
+{
+    static thread_local int nRecLevel = 0;
+    return nRecLevel;
+}
+
 class ERSProxyRasterBand final : public GDALProxyRasterBand
 {
 public:
@@ -850,6 +857,8 @@ public:
         eDataType = poUnderlyingBand->GetRasterDataType();
     }
 
+    int GetOverviewCount() override;
+
 protected:
     GDALRasterBand* RefUnderlyingRasterBand() override { return m_poUnderlyingBand; }
 
@@ -857,6 +866,16 @@ private:
     GDALRasterBand* m_poUnderlyingBand;
 };
 
+int ERSProxyRasterBand::GetOverviewCount()
+{
+    int& nRecLevel = GetRecLevel();
+    nRecLevel++;
+    int nRet = GDALProxyRasterBand::GetOverviewCount();
+    nRecLevel--;
+    return nRet;
+}
+
+
 } // namespace
 
 /************************************************************************/
@@ -866,6 +885,13 @@ private:
 GDALDataset *ERSDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
+    if( GetRecLevel() )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Attempt at recursively opening ERS dataset");
+        return nullptr;
+    }
+
     if( !Identify( poOpenInfo ) || poOpenInfo->fpL == nullptr )
         return nullptr;
 
@@ -996,28 +1022,30 @@ GDALDataset *ERSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( EQUAL(poHeader->Find("DataSetType",""),"Translated") )
     {
-        static thread_local int nRecLevel = 0;
-        if( nRecLevel == 0 )
-        {
-            nRecLevel ++;
-            poDS->poDepFile = (GDALDataset *)
-                GDALOpen( osDataFilePath, poOpenInfo->eAccess );
-            nRecLevel --;
+        int& nRecLevel = GetRecLevel();
+        nRecLevel++;
+        poDS->poDepFile = GDALDataset::FromHandle(
+            GDALOpen( osDataFilePath, poOpenInfo->eAccess ));
+        nRecLevel--;
 
-            if( poDS->poDepFile != nullptr
-                && poDS->poDepFile->GetRasterXSize() == poDS->GetRasterXSize()
-                && poDS->poDepFile->GetRasterYSize() == poDS->GetRasterYSize()
-                && poDS->poDepFile->GetRasterCount() >= nBands )
+        if( poDS->poDepFile != nullptr
+            && poDS->poDepFile->GetRasterXSize() == poDS->GetRasterXSize()
+            && poDS->poDepFile->GetRasterYSize() == poDS->GetRasterYSize()
+            && poDS->poDepFile->GetRasterCount() >= nBands )
+        {
+            for( int iBand = 0; iBand < nBands; iBand++ )
             {
-                for( int iBand = 0; iBand < nBands; iBand++ )
-                {
-                    // Assume pixel interleaved.
-                    poDS->SetBand( iBand+1,
-                        new ERSProxyRasterBand(
-                                poDS->poDepFile->GetRasterBand( iBand+1 )) );
-                }
+                // Assume pixel interleaved.
+                poDS->SetBand( iBand+1,
+                    new ERSProxyRasterBand(
+                            poDS->poDepFile->GetRasterBand( iBand+1 )) );
             }
         }
+        else
+        {
+            delete poDS->poDepFile;
+            poDS->poDepFile = nullptr;
+        }
     }
 
 /* ==================================================================== */
