@@ -40,6 +40,8 @@
 #include <memory>
 #include <vector>
 
+static bool gbHasLZW = false;
+
 extern "C" CPL_DLL void GDALRegister_COG();
 
 /************************************************************************/
@@ -164,7 +166,7 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
 
         // "Normalize" SRS as AUTH:CODE
         OGRSpatialReference oTargetSRS;
-        oTargetSRS.SetFromUserInput(osTargetSRS);
+        oTargetSRS.SetFromUserInput(osTargetSRS, OGRSpatialReference::SET_FROM_USER_INPUT_LIMITATIONS_get());
         const char* pszAuthCode = oTargetSRS.GetAuthorityCode(nullptr);
         const char* pszAuthName = oTargetSRS.GetAuthorityName(nullptr);
         if( pszAuthName && pszAuthCode )
@@ -180,7 +182,7 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
     void* hTransformArg = nullptr;
 
     OGRSpatialReference oTargetSRS;
-    oTargetSRS.SetFromUserInput(osTargetSRS);
+    oTargetSRS.SetFromUserInput(osTargetSRS, OGRSpatialReference::SET_FROM_USER_INPUT_LIMITATIONS_get());
     const char* pszAuthCode = oTargetSRS.GetAuthorityCode(nullptr);
     const int nEPSGCode = pszAuthCode ? atoi(pszAuthCode) : 0;
 
@@ -233,6 +235,10 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
                 {
                     hTransformArg = GDALCreateGenImgProjTransformer2(
                         GDALDataset::FromHandle(poTmpDS.get()), nullptr, aosTO.List() );
+                    if( hTransformArg == nullptr )
+                    {
+                        return false;
+                    }
                 }
             }
         }
@@ -241,6 +247,10 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
     {
         hTransformArg =
             GDALCreateGenImgProjTransformer2( poSrcDS, nullptr, aosTO.List() );
+        if( hTransformArg == nullptr )
+        {
+            return false;
+        }
     }
 
     GDALTransformerInfo* psInfo = static_cast<GDALTransformerInfo*>(hTransformArg);
@@ -752,7 +762,8 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
         }
     }
 
-    CPLString osCompress = CSLFetchNameValueDef(papszOptions, "COMPRESS", "NONE");
+    CPLString osCompress = CSLFetchNameValueDef(papszOptions, "COMPRESS",
+                                                gbHasLZW ? "LZW" : "NONE");
     if( EQUAL(osCompress, "JPEG") &&
         poCurDS->GetRasterCount() == 4 )
     {
@@ -983,12 +994,28 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
         aosOptions.SetNameValue("ZSTD_LEVEL",
                                 CSLFetchNameValue(papszOptions, "LEVEL"));
     }
+    else if( EQUAL(osCompress, "LZMA") )
+    {
+        aosOptions.SetNameValue("LZMA_PRESET",
+                                CSLFetchNameValue(papszOptions, "LEVEL"));
+    }
 
     if( STARTS_WITH_CI(osCompress, "LERC") )
     {
         aosOptions.SetNameValue("MAX_Z_ERROR",
                                 CSLFetchNameValue(papszOptions, "MAX_Z_ERROR"));
     }
+
+    if( STARTS_WITH_CI(osCompress, "JXL") )
+    {
+        aosOptions.SetNameValue("JXL_LOSSLESS",
+                                CSLFetchNameValue(papszOptions, "JXL_LOSSLESS"));
+        aosOptions.SetNameValue("JXL_EFFORT",
+                                CSLFetchNameValue(papszOptions, "JXL_EFFORT"));
+        aosOptions.SetNameValue("JXL_DISTANCE",
+                                CSLFetchNameValue(papszOptions, "JXL_DISTANCE"));
+    }
+
     aosOptions.SetNameValue("BIGTIFF",
                                 CSLFetchNameValue(papszOptions, "BIGTIFF"));
     aosOptions.SetNameValue("NUM_THREADS",
@@ -1027,14 +1054,15 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
                                      CPLSPrintf("%d", nAlignedLevels));
          }
     }
-    const char* pszOverviewCompress = CSLFetchNameValue(papszOptions, "OVERVIEW_COMPRESS");
+    const char* pszOverviewCompress = CSLFetchNameValueDef(
+        papszOptions, "OVERVIEW_COMPRESS", osCompress.c_str());
 
     CPLConfigOptionSetter ovrCompressSetter("COMPRESS_OVERVIEW", pszOverviewCompress, true);
     CPLConfigOptionSetter ovrQualityJpegSetter("JPEG_QUALITY_OVERVIEW", CSLFetchNameValue(papszOptions, "OVERVIEW_QUALITY"), true);
     CPLConfigOptionSetter ovrQualityWebpSetter("WEBP_LEVEL_OVERVIEW", CSLFetchNameValue(papszOptions, "OVERVIEW_QUALITY"), true);
 
     std::unique_ptr<CPLConfigOptionSetter> poPhotometricSetter;
-    if (pszOverviewCompress != nullptr && nBands == 3 && EQUAL(pszOverviewCompress, "JPEG") )
+    if (nBands == 3 && EQUAL(pszOverviewCompress, "JPEG") )
     {
         poPhotometricSetter.reset(new CPLConfigOptionSetter("PHOTOMETRIC_OVERVIEW", "YCBCR", true));
     }
@@ -1062,7 +1090,7 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
     GDALDestroyScaledProgress(pScaledProgress);
 
     if( poRet )
-        poRet->FlushCache();
+        poRet->FlushCache(false);
 
     CPLDebug("COG", "Generating final product: end");
     return poRet;
@@ -1129,6 +1157,7 @@ GDALCOGDriver::GDALCOGDriver()
     osCompressValues = GTiffGetCompressValues(
         bHasLZW, bHasDEFLATE, bHasLZMA, bHasZSTD, bHasJPEG, bHasWebP, bHasLERC,
         true /* bForCOG */);
+    gbHasLZW = bHasLZW;
 }
 
 void GDALCOGDriver::InitializeCreationOptionList()
@@ -1139,15 +1168,19 @@ void GDALCOGDriver::InitializeCreationOptionList()
 
     CPLString osOptions;
     osOptions = "<CreationOptionList>"
-                "   <Option name='COMPRESS' type='string-select'>";
+                "   <Option name='COMPRESS' type='string-select' default='";
+    osOptions += bHasLZW ? "LZW" : "NONE";
+    osOptions += "'>";
     osOptions += osCompressValues;
     osOptions += "   </Option>";
 
-    osOptions += "   <Option name='OVERVIEW_COMPRESS' type='string-select'>";
+    osOptions += "   <Option name='OVERVIEW_COMPRESS' type='string-select' default='";
+    osOptions += bHasLZW ? "LZW" : "NONE";
+    osOptions += "'>";
     osOptions += osCompressValues;
     osOptions += "   </Option>";
 
-    if( bHasLZW || bHasDEFLATE || bHasZSTD )
+    if( bHasLZW || bHasDEFLATE || bHasZSTD || bHasLZMA)
     {
         const char* osPredictorOptions =  "     <Value>YES</Value>"
                      "     <Value>NO</Value>"
@@ -1155,7 +1188,7 @@ void GDALCOGDriver::InitializeCreationOptionList()
                      "     <Value alias='3'>FLOATING_POINT</Value>";
 
         osOptions += "   <Option name='LEVEL' type='int' "
-            "description='DEFLATE/ZSTD compression level: 1 (fastest)'/>";
+            "description='DEFLATE/ZSTD/LZMA compression level: 1 (fastest)'/>";
 
         osOptions += "   <Option name='PREDICTOR' type='string-select' default='FALSE'>";
         osOptions += osPredictorOptions;
@@ -1176,6 +1209,12 @@ void GDALCOGDriver::InitializeCreationOptionList()
         osOptions += ""
 "   <Option name='MAX_Z_ERROR' type='float' description='Maximum error for LERC compression' default='0'/>";
     }
+#ifdef HAVE_JXL
+    osOptions += ""
+"   <Option name='JXL_LOSSLESS' type='boolean' description='Whether JPEGXL compression should be lossless' default='YES'/>"
+"   <Option name='JXL_EFFORT' type='int' description='Level of effort 1(fast)-9(slow)' default='5'/>"
+"   <Option name='JXL_DISTANCE' type='float' description='Distance level for lossy compression (0=mathematically lossless, 1.0=visually lossless, usual range [0.5,3])' default='1.0' min='0.1' max='15.0'/>";
+#endif
     osOptions +=
 "   <Option name='NUM_THREADS' type='string' "
         "description='Number of worker threads for compression. "
@@ -1273,6 +1312,8 @@ void GDALRegister_COG()
 
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
+    poDriver->SetMetadataItem( GDAL_DCAP_COORDINATE_EPOCH, "YES" );
+
     poDriver->pfnCreateCopy = COGCreateCopy;
 
     GetGDALDriverManager()->RegisterDriver( poDriver );
