@@ -34,7 +34,7 @@
 
 #include <vector>
 
-CPL_CVSID("$Id: cpgdataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id: cpgdataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 enum Interleave { BSQ, BIL, BIP };
 
@@ -133,7 +133,7 @@ CPGDataset::CPGDataset() :
 CPGDataset::~CPGDataset()
 
 {
-    FlushCache();
+    FlushCache(true);
 
     for( int iBand = 0; iBand < 4; iBand++ )
     {
