@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2002, Frank Warmerdam
- * Copyright (c) 2007-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2007-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -55,7 +55,7 @@
 #include "ogr_spatialref.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: envidataset.cpp 558f8a7aa15498ac25fb4a8227b220c1d4fecf29 2019-09-14 12:13:43 +0200 Even Rouault $")
+CPL_CVSID("$Id: envidataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 // TODO(schwehr): This really should be defined in port/somewhere.h.
 constexpr double kdfDegToRad = M_PI / 180.0;
@@ -250,7 +250,6 @@ ENVIDataset::ENVIDataset() :
     bFoundMapinfo(false),
     bHeaderDirty(false),
     bFillFile(false),
-    pszProjection(CPLStrdup("")),
     interleave(BSQ)
 {
     adfGeoTransform[0] = 0.0;
@@ -268,11 +267,11 @@ ENVIDataset::ENVIDataset() :
 ENVIDataset::~ENVIDataset()
 
 {
-    ENVIDataset::FlushCache();
+    ENVIDataset::FlushCache(true);
     if( fpImage )
     {
         // Make sure the binary file has the expected size
-        if( bFillFile && nBands > 0)
+        if( !bSuppressOnClose && bFillFile && nBands > 0)
         {
             const int nDataSize =
                 GDALGetDataTypeSizeBytes(GetRasterBand(1)->GetRasterDataType());
@@ -309,7 +308,6 @@ ENVIDataset::~ENVIDataset()
     {
         GDALDeinitGCPs(static_cast<int>(m_asGCPs.size()), m_asGCPs.data());
     }
-    CPLFree(pszProjection);
     CPLFree(pszHDRFilename);
 }
 
@@ -317,14 +315,14 @@ ENVIDataset::~ENVIDataset()
 /*                             FlushCache()                             */
 /************************************************************************/
 
-void ENVIDataset::FlushCache()
+void ENVIDataset::FlushCache(bool bAtClosing)
 
 {
-    RawDataset::FlushCache();
+    RawDataset::FlushCache(bAtClosing);
 
     GDALRasterBand *band = GetRasterCount() > 0 ? GetRasterBand(1) : nullptr;
 
-    if ( band == nullptr || !bHeaderDirty )
+    if ( band == nullptr || !bHeaderDirty || (bAtClosing && bSuppressOnClose) )
         return;
 
     // If opening an existing file in Update mode (i.e. "r+") we need to make
@@ -458,6 +456,13 @@ void ENVIDataset::FlushCache()
     }
     bOK &= VSIFPrintfL(fp, "}\n") >= 0;
 
+    int bHasNoData = FALSE;
+    double dfNoDataValue = band->GetNoDataValue(&bHasNoData);
+    if( bHasNoData )
+    {
+        bOK &= VSIFPrintfL(fp, "data ignore value = %.18g\n", dfNoDataValue) >= 0;
+    }
+
     // Write the metadata that was read into the ENVI domain.
     char **papszENVIMetadata = GetMetadata("ENVI");
 
@@ -497,7 +502,8 @@ void ENVIDataset::FlushCache()
              poKey == "class names" ||
              poKey == "band names" ||
              poKey == "map info" ||
-             poKey == "projection info" )
+             poKey == "projection info" ||
+             poKey == "data ignore value" )
         {
             CSLDestroy(papszTokens);
             continue;
@@ -542,7 +548,7 @@ char **ENVIDataset::GetFileList()
 /*      TODO: We really need to do some name lookups.                   */
 /************************************************************************/
 
-static int ENVIGetEPSGGeogCS( OGRSpatialReference *poThis )
+static int ENVIGetEPSGGeogCS( const OGRSpatialReference *poThis )
 
 {
     const char *pszAuthName = poThis->GetAuthorityName("GEOGCS");
@@ -620,7 +626,12 @@ void ENVIDataset::WriteProjectionInfo()
         adfGeoTransform[0] != 0.0 || adfGeoTransform[1] != 1.0 ||
         adfGeoTransform[2] != 0.0 || adfGeoTransform[3] != 0.0 ||
         adfGeoTransform[4] != 0.0 || adfGeoTransform[5] != 1.0;
-    if( bHasNonDefaultGT )
+    if( adfGeoTransform[1] > 0.0 && adfGeoTransform[2] == 0.0 &&
+        adfGeoTransform[4] == 0.0 && adfGeoTransform[5] > 0.0 )
+    {
+        osRotation = ", rotation=180";
+    }
+    else if( bHasNonDefaultGT )
     {
         const double dfRotation1 =
             -atan2(-adfGeoTransform[2], adfGeoTransform[1]) * kdfRadToDeg;
@@ -647,10 +658,7 @@ void ENVIDataset::WriteProjectionInfo()
 
     // Minimal case - write out simple geotransform if we have a
     // non-default geotransform.
-    const std::string osLocalCs = "LOCAL_CS";
-    if( pszProjection == nullptr || strlen(pszProjection) == 0  ||
-        (strlen(pszProjection) >= osLocalCs.size() &&
-         STARTS_WITH(pszProjection, osLocalCs.c_str())) )
+    if( m_oSRS.IsEmpty() || m_oSRS.IsLocal() )
     {
         if( bHasNonDefaultGT )
         {
@@ -663,12 +671,8 @@ void ENVIDataset::WriteProjectionInfo()
         return;
     }
 
-    // Ingest WKT.
-    OGRSpatialReference oSRS;
-    if( oSRS.importFromWkt(pszProjection) != OGRERR_NONE )
-        return;
-
     // Try to translate the datum and get major/minor ellipsoid values.
+    const OGRSpatialReference& oSRS = m_oSRS;
     const int nEPSG_GCS = ENVIGetEPSGGeogCS(&oSRS);
     CPLString osDatum;
 
@@ -954,18 +958,16 @@ void ENVIDataset::WriteProjectionInfo()
     }
 
     // write out coordinate system string
-    if ( oSRS.morphToESRI() == OGRERR_NONE )
+    char *pszProjESRI = nullptr;
+    const char* const apszOptions[] = { "FORMAT=WKT1_ESRI", nullptr };
+    if ( oSRS.exportToWkt(&pszProjESRI, apszOptions) == OGRERR_NONE )
     {
-        char *pszProjESRI = nullptr;
-        if ( oSRS.exportToWkt(&pszProjESRI) == OGRERR_NONE )
-        {
-            if ( strlen(pszProjESRI) )
-                bOK &= VSIFPrintfL(fp, "coordinate system string = {%s}\n",
-                                   pszProjESRI) >= 0;
-        }
-        CPLFree(pszProjESRI);
-        pszProjESRI = nullptr;
+        if ( strlen(pszProjESRI) )
+            bOK &= VSIFPrintfL(fp, "coordinate system string = {%s}\n",
+                               pszProjESRI) >= 0;
     }
+    CPLFree(pszProjESRI);
+    pszProjESRI = nullptr;
 
     if( !bOK )
     {
@@ -1140,20 +1142,24 @@ bool ENVIDataset::WritePseudoGcpInfo()
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                          GetSpatialRef()                             */
 /************************************************************************/
 
-const char *ENVIDataset::_GetProjectionRef() { return pszProjection; }
+const OGRSpatialReference *ENVIDataset::GetSpatialRef() const
+{
+    return m_oSRS.IsEmpty() ? nullptr : &m_oSRS;
+}
 
 /************************************************************************/
-/*                          SetProjection()                             */
+/*                          SetSpatialRef()                             */
 /************************************************************************/
 
-CPLErr ENVIDataset::_SetProjection( const char *pszNewProjection )
+CPLErr ENVIDataset::SetSpatialRef( const OGRSpatialReference *poSRS )
 
 {
-    CPLFree(pszProjection);
-    pszProjection = CPLStrdup(pszNewProjection);
+    m_oSRS.Clear();
+    if( poSRS )
+        m_oSRS = *poSRS;
 
     bHeaderDirty = true;
 
@@ -1376,6 +1382,7 @@ bool ENVIDataset::ProcessMapinfo( const char *pszMapinfo )
     char **papszFields = SplitList(pszMapinfo);
     const char *pszUnits = nullptr;
     double dfRotation = 0.0;
+    bool bUpsideDown = false;
     const int nCount = CSLCount(papszFields);
 
     if( nCount < 7 )
@@ -1394,8 +1401,9 @@ bool ENVIDataset::ProcessMapinfo( const char *pszMapinfo )
         else if ( STARTS_WITH(papszFields[i], "rotation=") )
         {
             dfRotation =
-                CPLAtof(papszFields[i] + strlen("rotation=")) *
-                kdfDegToRad * -1.0;
+                CPLAtof(papszFields[i] + strlen("rotation="));
+            bUpsideDown = fabs(dfRotation) == 180.0;
+            dfRotation *= kdfDegToRad * -1.0;
         }
     }
 
@@ -1432,6 +1440,13 @@ bool ENVIDataset::ProcessMapinfo( const char *pszMapinfo )
     adfGeoTransform[3] = pixelNorthing + (yReference - 1) * yPixelSize;
     adfGeoTransform[4] = -sin(dfRotation) * yPixelSize;
     adfGeoTransform[5] = -cos(dfRotation) * yPixelSize;
+    if( bUpsideDown ) // to avoid numeric approximations
+    {
+        adfGeoTransform[1] = xPixelSize;
+        adfGeoTransform[2] = 0;
+        adfGeoTransform[4] = 0;
+        adfGeoTransform[5] = yPixelSize;
+    }
 
     // TODO(schwehr): Symbolic constants for the fields.
     // Capture projection.
@@ -1544,7 +1559,7 @@ bool ENVIDataset::ProcessMapinfo( const char *pszMapinfo )
     // Still lots more that could be added for someone with the patience.
 
     // Fallback to localcs if we don't recognise things.
-    if( oSRS.GetRoot() == nullptr )
+    if( oSRS.IsEmpty() )
         oSRS.SetLocalCS(papszFields[0]);
 
     // Try to set datum from projection info line if we have a
@@ -1622,16 +1637,8 @@ bool ENVIDataset::ProcessMapinfo( const char *pszMapinfo )
         }
     }
 
-    // Turn back into WKT.
-    if( oSRS.GetRoot() != nullptr )
-    {
-        if ( pszProjection )
-        {
-            CPLFree(pszProjection);
-            pszProjection = nullptr;
-        }
-        oSRS.exportToWkt(&pszProjection);
-    }
+    m_oSRS = oSRS;
+    m_oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
 
     CSLDestroy(papszFields);
     CSLDestroy(papszPI);
@@ -1821,6 +1828,12 @@ void ENVIDataset::ProcessGeoPoints( const char *pszGeoPoints )
     CSLDestroy(papszFields);
 }
 
+static unsigned byteSwapUInt(unsigned swapMe)
+{
+    CPL_MSBPTR32(&swapMe);
+    return swapMe;
+}
+
 void ENVIDataset::ProcessStatsFile()
 {
     osStaFilename = CPLResetExtension(pszHDRFilename, "sta");
@@ -1853,10 +1866,10 @@ void ENVIDataset::ProcessStatsFile()
     }
 
     // TODO(schwehr): What are 1, 4, 8, and 40?
-    int lOffset = 0;
+    unsigned lOffset = 0;
     if( VSIFSeekL(fpStaFile, 40 + static_cast<vsi_l_offset>(nb + 1) * 4, SEEK_SET) == 0 &&
-        VSIFReadL(&lOffset, sizeof(int), 1, fpStaFile) == 1 &&
-        VSIFSeekL(fpStaFile, 40 + static_cast<vsi_l_offset>(nb + 1) * 8 + byteSwapInt(lOffset) + nb,
+        VSIFReadL(&lOffset, sizeof(lOffset), 1, fpStaFile) == 1 &&
+        VSIFSeekL(fpStaFile, 40 + static_cast<vsi_l_offset>(nb + 1) * 8 + byteSwapUInt(lOffset) + nb,
                   SEEK_SET) == 0)
     {
         // This should be the beginning of the statistics.
@@ -1991,6 +2004,22 @@ bool ENVIDataset::ReadHeader( VSILFILE *fpHdr )
     return true;
 }
 
+/************************************************************************/
+/*                        GetRawBinaryLayout()                          */
+/************************************************************************/
+
+bool ENVIDataset::GetRawBinaryLayout(GDALDataset::RawBinaryLayout& sLayout)
+{
+    const bool bIsCompressed = atoi(
+        m_aosHeader.FetchNameValueDef("file_compression", "0")) != 0;
+    if( bIsCompressed )
+        return false;
+    if( !RawDataset::GetRawBinaryLayout(sLayout) )
+        return false;
+    sLayout.osRawFilename = GetDescription();
+    return true;
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -2000,7 +2029,7 @@ GDALDataset *ENVIDataset::Open( GDALOpenInfo *poOpenInfo )
     return Open(poOpenInfo, true);
 }
 
-GDALDataset *ENVIDataset::Open( GDALOpenInfo *poOpenInfo, bool bFileSizeCheck )
+ENVIDataset *ENVIDataset::Open( GDALOpenInfo *poOpenInfo, bool bFileSizeCheck )
 
 {
     // Assume the caller is pointing to the binary (i.e. .bil) file.
@@ -2022,28 +2051,31 @@ GDALDataset *ENVIDataset::Open( GDALOpenInfo *poOpenInfo, bool bFileSizeCheck )
     char **papszSiblingFiles = poOpenInfo->GetSiblingFiles();
     if (papszSiblingFiles == nullptr)
     {
-        osHdrFilename = CPLResetExtension(poOpenInfo->pszFilename, "hdr");
+        // First try hdr as an extra extension
+        osHdrFilename =
+            CPLFormFilename(nullptr, poOpenInfo->pszFilename, "hdr");
         fpHeader = VSIFOpenL(osHdrFilename, pszMode);
 
-        if( fpHeader == nullptr && VSIIsCaseSensitiveFS(osHdrFilename) )
-        {
-            osHdrFilename = CPLResetExtension(poOpenInfo->pszFilename, "HDR");
-            fpHeader = VSIFOpenL(osHdrFilename, pszMode);
-        }
-
-        if( fpHeader == nullptr )
-        {
-            osHdrFilename =
-                CPLFormFilename(nullptr, poOpenInfo->pszFilename, "hdr");
-            fpHeader = VSIFOpenL(osHdrFilename, pszMode);
-        }
-
         if( fpHeader == nullptr && VSIIsCaseSensitiveFS(osHdrFilename) )
         {
             osHdrFilename =
                 CPLFormFilename(nullptr, poOpenInfo->pszFilename, "HDR");
             fpHeader = VSIFOpenL(osHdrFilename, pszMode);
         }
+
+        // Otherwise, try .hdr as a replacement extension
+        if( fpHeader == nullptr )
+        {
+            osHdrFilename = CPLResetExtension(poOpenInfo->pszFilename, "hdr");
+            fpHeader = VSIFOpenL(osHdrFilename, pszMode);
+        }
+
+        if( fpHeader == nullptr && VSIIsCaseSensitiveFS(osHdrFilename) )
+        {
+            osHdrFilename = CPLResetExtension(poOpenInfo->pszFilename, "HDR");
+            fpHeader = VSIFOpenL(osHdrFilename, pszMode);
+        }
+
     }
     else
     {
@@ -2051,25 +2083,22 @@ GDALDataset *ENVIDataset::Open( GDALOpenInfo *poOpenInfo, bool bFileSizeCheck )
         CPLString osPath = CPLGetPath(poOpenInfo->pszFilename);
         CPLString osName = CPLGetFilename(poOpenInfo->pszFilename);
 
-        int iFile =
-            CSLFindString(papszSiblingFiles, CPLResetExtension(osName, "hdr"));
+        // First try hdr as an extra extension
+        int iFile = CSLFindString(papszSiblingFiles,
+                                  CPLFormFilename(nullptr, osName, "hdr"));
+        if( iFile < 0 )
+        {
+            // Otherwise, try .hdr as a replacement extension
+            iFile =
+                CSLFindString(papszSiblingFiles, CPLResetExtension(osName, "hdr"));
+        }
+
         if( iFile >= 0 )
         {
             osHdrFilename =
                 CPLFormFilename(osPath, papszSiblingFiles[iFile], nullptr);
             fpHeader = VSIFOpenL(osHdrFilename, pszMode);
         }
-        else
-        {
-            iFile = CSLFindString(papszSiblingFiles,
-                                  CPLFormFilename(nullptr, osName, "hdr"));
-            if( iFile >= 0 )
-            {
-                osHdrFilename =
-                    CPLFormFilename(osPath, papszSiblingFiles[iFile], nullptr);
-                fpHeader = VSIFOpenL(osHdrFilename, pszMode);
-            }
-        }
     }
 
     if( fpHeader == nullptr )
@@ -2730,7 +2759,7 @@ GDALDataset *ENVIDataset::Create( const char *pszFilename,
         return nullptr;
 
     GDALOpenInfo oOpenInfo(pszFilename, GA_Update);
-    ENVIDataset *poDS = reinterpret_cast<ENVIDataset *>(Open(&oOpenInfo, false));
+    ENVIDataset *poDS = Open(&oOpenInfo, false);
     if( poDS )
     {
         poDS->SetFillFile();
@@ -2771,6 +2800,16 @@ CPLErr ENVIRasterBand::SetCategoryNames( char **papszCategoryNamesIn )
     return RawRasterBand::SetCategoryNames(papszCategoryNamesIn);
 }
 
+/************************************************************************/
+/*                            SetNoDataValue()                          */
+/************************************************************************/
+
+CPLErr ENVIRasterBand::SetNoDataValue( double dfNoDataValue )
+{
+    reinterpret_cast<ENVIDataset *>(poDS)->bHeaderDirty = true;
+    return RawRasterBand::SetNoDataValue(dfNoDataValue);
+}
+
 /************************************************************************/
 /*                         GDALRegister_ENVI()                          */
 /************************************************************************/
@@ -2785,7 +2824,7 @@ void GDALRegister_ENVI()
     poDriver->SetDescription("ENVI");
     poDriver->SetMetadataItem(GDAL_DCAP_RASTER, "YES");
     poDriver->SetMetadataItem(GDAL_DMD_LONGNAME, "ENVI .hdr Labelled");
-    poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "frmt_various.html#ENVI");
+    poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "drivers/raster/envi.html");
     poDriver->SetMetadataItem(GDAL_DMD_EXTENSION, "");
     poDriver->SetMetadataItem(GDAL_DMD_CREATIONDATATYPES,
                               "Byte Int16 UInt16 Int32 UInt32 "
