@@ -33,7 +33,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: idadataset.cpp f6099e5ed704166bf5cc113a053dd1b2725cb391 2020-03-22 11:20:10 +0100 Kai Pastor $")
+CPL_CVSID("$Id: idadataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                                tp2c()                                */
@@ -151,7 +151,7 @@ class IDADataset final: public RawDataset
     IDADataset();
     ~IDADataset() override;
 
-    void FlushCache() override;
+    void FlushCache(bool bAtClosing) override;
 
     const OGRSpatialReference* GetSpatialRef() const override;
     CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override;
@@ -163,7 +163,7 @@ class IDADataset final: public RawDataset
     static GDALDataset *Create( const char * pszFilename,
                                 int nXSize, int nYSize, int nBands,
                                 GDALDataType eType,
-                                char ** /* papszParmList */ );
+                                char ** /* papszParamList */ );
 };
 
 /************************************************************************/
@@ -391,7 +391,7 @@ IDADataset::IDADataset() :
 IDADataset::~IDADataset()
 
 {
-    IDADataset::FlushCache();
+    IDADataset::FlushCache(true);
 
     if( fpRaw != nullptr )
     {
@@ -471,10 +471,10 @@ void IDADataset::ProcessGeoref()
 /*                             FlushCache()                             */
 /************************************************************************/
 
-void IDADataset::FlushCache()
+void IDADataset::FlushCache(bool bAtClosing)
 
 {
-    RawDataset::FlushCache();
+    RawDataset::FlushCache(bAtClosing);
 
     if( bHeaderDirty )
     {
@@ -796,6 +796,9 @@ GDALDataset *IDADataset::Open( GDALOpenInfo * poOpenInfo )
     if( nActualFileSize != nExpectedFileSize )
         return nullptr;
 
+    if( !GDALIsDriverDeprecatedForGDAL35StillEnabled("IDA") )
+        return nullptr;
+
 /* -------------------------------------------------------------------- */
 /*      Create the dataset.                                             */
 /* -------------------------------------------------------------------- */
@@ -989,9 +992,12 @@ GDALDataset *IDADataset::Open( GDALOpenInfo * poOpenInfo )
 GDALDataset *IDADataset::Create( const char * pszFilename,
                                  int nXSize, int nYSize, int nBands,
                                  GDALDataType eType,
-                                 char ** /* papszParmList */ )
+                                 char ** /* papszParamList */ )
 
 {
+    if( !GDALIsDriverDeprecatedForGDAL35StillEnabled("IDA") )
+        return nullptr;
+
 /* -------------------------------------------------------------------- */
 /*      Verify input options.                                           */
 /* -------------------------------------------------------------------- */
