@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2007-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2007-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -47,7 +47,7 @@
 #include "ogr_core.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: gdalpamdataset.cpp ec7b85e6bb8f9737693a31f0bf7166e31e10992e 2018-04-16 00:08:36 +0200 Even Rouault $")
+CPL_CVSID("$Id: gdalpamdataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                           GDALPamDataset()                           */
@@ -140,9 +140,7 @@ CPL_CVSID("$Id: gdalpamdataset.cpp ec7b85e6bb8f9737693a31f0bf7166e31e10992e 2018
  */
 class GDALPamDataset;
 
-GDALPamDataset::GDALPamDataset() :
-    nPamFlags(0),
-    psPam(nullptr)
+GDALPamDataset::GDALPamDataset()
 {
     SetMOFlags( GetMOFlags() | GMO_PAM_CLASS );
 }
@@ -157,7 +155,7 @@ GDALPamDataset::~GDALPamDataset()
     if( nPamFlags & GPF_DIRTY )
     {
         CPLDebug( "GDALPamDataset", "In destructor with dirty metadata." );
-        FlushCache();
+        GDALPamDataset::TrySaveXML();
     }
 
     PamClear();
@@ -167,10 +165,10 @@ GDALPamDataset::~GDALPamDataset()
 /*                             FlushCache()                             */
 /************************************************************************/
 
-void GDALPamDataset::FlushCache()
+void GDALPamDataset::FlushCache(bool bAtClosing)
 
 {
-    GDALDataset::FlushCache();
+    GDALDataset::FlushCache(bAtClosing);
     if( nPamFlags & GPF_DIRTY )
         TrySaveXML();
 }
@@ -194,8 +192,46 @@ CPLXMLNode *GDALPamDataset::SerializeToXML( const char *pszUnused )
 /* -------------------------------------------------------------------- */
 /*      SRS                                                             */
 /* -------------------------------------------------------------------- */
-    if( psPam->pszProjection != nullptr && strlen(psPam->pszProjection) > 0 )
-        CPLSetXMLValue( psDSTree, "SRS", psPam->pszProjection );
+    if( psPam->poSRS && !psPam->poSRS->IsEmpty() )
+    {
+        char* pszWKT = nullptr;
+        {
+            CPLErrorStateBackuper oErrorStateBackuper;
+            CPLErrorHandlerPusher oErrorHandler(CPLQuietErrorHandler);
+            if( psPam->poSRS->exportToWkt(&pszWKT) != OGRERR_NONE )
+            {
+                CPLFree(pszWKT);
+                pszWKT = nullptr;
+                const char* const apszOptions[] = { "FORMAT=WKT2", nullptr };
+                psPam->poSRS->exportToWkt(&pszWKT, apszOptions);
+            }
+        }
+        CPLXMLNode* psSRSNode = CPLCreateXMLElementAndValue( psDSTree, "SRS", pszWKT );
+        CPLFree(pszWKT);
+        const auto& mapping = psPam->poSRS->GetDataAxisToSRSAxisMapping();
+        CPLString osMapping;
+        for( size_t i = 0; i < mapping.size(); ++i )
+        {
+            if( !osMapping.empty() )
+                osMapping += ",";
+            osMapping += CPLSPrintf("%d", mapping[i]);
+        }
+        CPLAddXMLAttributeAndValue(psSRSNode, "dataAxisToSRSAxisMapping",
+                                   osMapping.c_str());
+
+        const double dfCoordinateEpoch = psPam->poSRS->GetCoordinateEpoch();
+        if( dfCoordinateEpoch > 0 )
+        {
+            std::string osCoordinateEpoch = CPLSPrintf("%f", dfCoordinateEpoch);
+            if( osCoordinateEpoch.find('.') != std::string::npos )
+            {
+                while( osCoordinateEpoch.back() == '0' )
+                    osCoordinateEpoch.resize(osCoordinateEpoch.size()-1);
+            }
+            CPLAddXMLAttributeAndValue(psSRSNode, "coordinateEpoch",
+                                       osCoordinateEpoch.c_str());
+        }
+    }
 
 /* -------------------------------------------------------------------- */
 /*      GeoTransform.                                                   */
@@ -233,7 +269,7 @@ CPLXMLNode *GDALPamDataset::SerializeToXML( const char *pszUnused )
         GDALSerializeGCPListToXML( psDSTree,
                                    psPam->pasGCPList,
                                    psPam->nGCPCount,
-                                   psPam->pszGCPProjection );
+                                   psPam->poGCP_SRS );
     }
 
 /* -------------------------------------------------------------------- */
@@ -249,14 +285,13 @@ CPLXMLNode *GDALPamDataset::SerializeToXML( const char *pszUnused )
 
     for( int iBand = 0; iBand < GetRasterCount(); iBand++ )
     {
-        GDALPamRasterBand * const poBand =
-            cpl::down_cast<GDALPamRasterBand *>(
-                GetRasterBand(iBand+1) );
+        GDALRasterBand * const poBand = GetRasterBand(iBand+1);
 
         if( poBand == nullptr || !(poBand->GetMOFlags() & GMO_PAM_CLASS) )
             continue;
 
-        CPLXMLNode * const psBandTree = poBand->SerializeToXML( pszUnused );
+        CPLXMLNode * const psBandTree =
+            cpl::down_cast<GDALPamRasterBand *>(poBand)->SerializeToXML( pszUnused );
 
         if( psBandTree != nullptr )
         {
@@ -313,14 +348,6 @@ void GDALPamDataset::PamInitialize()
         nPamFlags |= GPF_AUXMODE;
 
     psPam = new GDALDatasetPamInfo;
-    psPam->pszPamFilename = nullptr;
-    psPam->pszProjection = nullptr;
-    psPam->bHaveGeoTransform = FALSE;
-    psPam->nGCPCount = 0;
-    psPam->pasGCPList = nullptr;
-    psPam->pszGCPProjection = nullptr;
-    psPam->bHasMetadata = FALSE;
-
     for( int iBand = 0; iBand < GetRasterCount(); iBand++ )
     {
         GDALRasterBand *poBand = GetRasterBand(iBand+1);
@@ -342,8 +369,10 @@ void GDALPamDataset::PamClear()
     if( psPam )
     {
         CPLFree( psPam->pszPamFilename );
-        CPLFree( psPam->pszProjection );
-        CPLFree( psPam->pszGCPProjection );
+        if( psPam->poSRS )
+            psPam->poSRS->Release();
+        if( psPam->poGCP_SRS )
+            psPam->poGCP_SRS->Release();
         if( psPam->nGCPCount > 0 )
         {
             GDALDeinitGCPs( psPam->nGCPCount, psPam->pasGCPList );
@@ -365,15 +394,36 @@ CPLErr GDALPamDataset::XMLInit( CPLXMLNode *psTree, const char *pszUnused )
 /* -------------------------------------------------------------------- */
 /*      Check for an SRS node.                                          */
 /* -------------------------------------------------------------------- */
-    if( strlen(CPLGetXMLValue(psTree, "SRS", "")) > 0 )
+    CPLXMLNode* psSRSNode = CPLGetXMLNode(psTree, "SRS");
+    if( psSRSNode )
     {
-        CPLFree( psPam->pszProjection );
-        psPam->pszProjection = nullptr;
+        if( psPam->poSRS )
+            psPam->poSRS->Release();
+        psPam->poSRS = new OGRSpatialReference();
+        psPam->poSRS->SetFromUserInput( CPLGetXMLValue(psSRSNode, nullptr, ""),
+                                        OGRSpatialReference::SET_FROM_USER_INPUT_LIMITATIONS );
+        const char* pszMapping =
+            CPLGetXMLValue(psSRSNode, "dataAxisToSRSAxisMapping", nullptr);
+        if( pszMapping )
+        {
+            char** papszTokens = CSLTokenizeStringComplex( pszMapping, ",", FALSE, FALSE);
+            std::vector<int> anMapping;
+            for( int i = 0; papszTokens && papszTokens[i]; i++ )
+            {
+                anMapping.push_back(atoi(papszTokens[i]));
+            }
+            CSLDestroy(papszTokens);
+            psPam->poSRS->SetDataAxisToSRSAxisMapping(anMapping);
+        }
+        else
+        {
+            psPam->poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+        }
 
-        OGRSpatialReference oSRS;
-        if( oSRS.SetFromUserInput( CPLGetXMLValue(psTree, "SRS", "") )
-            == OGRERR_NONE )
-            oSRS.exportToWkt( &(psPam->pszProjection) );
+        const char* pszCoordinateEpoch =
+            CPLGetXMLValue(psSRSNode, "coordinateEpoch", nullptr);
+        if( pszCoordinateEpoch )
+            psPam->poSRS->SetCoordinateEpoch(CPLAtof(pszCoordinateEpoch));
     }
 
 /* -------------------------------------------------------------------- */
@@ -407,8 +457,9 @@ CPLErr GDALPamDataset::XMLInit( CPLXMLNode *psTree, const char *pszUnused )
 
     if( psGCPList != nullptr )
     {
-        CPLFree( psPam->pszGCPProjection );
-        psPam->pszGCPProjection = nullptr;
+        if( psPam->poGCP_SRS )
+            psPam->poGCP_SRS->Release();
+        psPam->poGCP_SRS = nullptr;
 
         // Make sure any previous GCPs, perhaps from an .aux file, are cleared
         // if we have new ones.
@@ -423,18 +474,21 @@ CPLErr GDALPamDataset::XMLInit( CPLXMLNode *psTree, const char *pszUnused )
         GDALDeserializeGCPListFromXML( psGCPList,
                                        &(psPam->pasGCPList),
                                        &(psPam->nGCPCount),
-                                       &(psPam->pszGCPProjection) );
+                                       &(psPam->poGCP_SRS) );
     }
 
 /* -------------------------------------------------------------------- */
 /*      Apply any dataset level metadata.                               */
 /* -------------------------------------------------------------------- */
-    oMDMD.XMLInit( psTree, TRUE );
+    if( oMDMD.XMLInit( psTree, TRUE ) )
+    {
+        psPam->bHasMetadata = TRUE;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Try loading ESRI xml encoded GeodataXform.                      */
 /* -------------------------------------------------------------------- */
-    if (psPam->pszProjection == nullptr)
+    if (psPam->poSRS == nullptr)
     {
         // ArcGIS 9.3: GeodataXform as a root element
         CPLXMLNode* psGeodataXform = CPLGetXMLNode(psTree, "=GeodataXform");
@@ -465,18 +519,63 @@ CPLErr GDALPamDataset::XMLInit( CPLXMLNode *psTree, const char *pszUnused )
                                 "SpatialReference.WKT", nullptr);
             if (pszESRI_WKT)
             {
-                OGRSpatialReference* poSRS = new OGRSpatialReference(nullptr);
-                if (poSRS->importFromWkt(pszESRI_WKT) == OGRERR_NONE &&
-                    poSRS->morphFromESRI() == OGRERR_NONE)
+                psPam->poSRS = new OGRSpatialReference(nullptr);
+                psPam->poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+                if( psPam->poSRS->importFromWkt(pszESRI_WKT) != OGRERR_NONE)
                 {
-                    char* pszWKT = nullptr;
-                    if (poSRS->exportToWkt(&pszWKT) == OGRERR_NONE)
+                    delete psPam->poSRS;
+                    psPam->poSRS = nullptr;
+                }
+            }
+
+            // Parse GCPs
+            const CPLXMLNode* psSourceGCPS = CPLGetXMLNode(psGeodataXform, "SourceGCPs");
+            const CPLXMLNode* psTargetGCPs = CPLGetXMLNode(psGeodataXform, "TargetGCPs");
+            if( psSourceGCPS && psTargetGCPs && !psPam->bHaveGeoTransform )
+            {
+                std::vector<double> adfSource;
+                std::vector<double> adfTarget;
+                bool ySourceAllNegative = true;
+                for( auto psIter = psSourceGCPS->psChild; psIter; psIter = psIter->psNext )
+                {
+                    if( psIter->eType == CXT_Element && strcmp(psIter->pszValue, "Double") == 0 )
                     {
-                        psPam->pszProjection = CPLStrdup(pszWKT);
+                        adfSource.push_back(CPLAtof(CPLGetXMLValue(psIter, nullptr, "0")));
+                        if( (adfSource.size() % 2) == 0 && adfSource.back() > 0 )
+                            ySourceAllNegative = false;
                     }
-                    CPLFree(pszWKT);
+                }
+                for( auto psIter = psTargetGCPs->psChild; psIter; psIter = psIter->psNext )
+                {
+                    if( psIter->eType == CXT_Element && strcmp(psIter->pszValue, "Double") == 0 )
+                    {
+                        adfTarget.push_back(CPLAtof(CPLGetXMLValue(psIter, nullptr, "0")));
+                    }
+                }
+                if( !adfSource.empty() &&
+                    adfSource.size() == adfTarget.size() &&
+                    (adfSource.size() % 2) == 0 )
+                {
+                    std::vector<GDAL_GCP> asGCPs;
+                    asGCPs.resize(adfSource.size() / 2);
+                    char szEmptyString[] = { 0 };
+                    for( size_t i = 0; i+1 < adfSource.size(); i+= 2 )
+                    {
+                        asGCPs[i/2].pszId = szEmptyString;
+                        asGCPs[i/2].pszInfo = szEmptyString;
+                        asGCPs[i/2].dfGCPPixel = adfSource[i];
+                        asGCPs[i/2].dfGCPLine =
+                            ySourceAllNegative ? -adfSource[i+1] : adfSource[i+1];
+                        asGCPs[i/2].dfGCPX = adfTarget[i];
+                        asGCPs[i/2].dfGCPY = adfTarget[i+1];
+                        asGCPs[i/2].dfGCPZ = 0;
+                    }
+                    GDALPamDataset::SetGCPs( static_cast<int>(asGCPs.size()),
+                                             &asGCPs[0],
+                                             psPam->poSRS );
+                    delete psPam->poSRS;
+                    psPam->poSRS = nullptr;
                 }
-                delete poSRS;
             }
         }
         if( psValueAsXML )
@@ -499,13 +598,31 @@ CPLErr GDALPamDataset::XMLInit( CPLXMLNode *psTree, const char *pszUnused )
         if( nBand < 1 || nBand > GetRasterCount() )
             continue;
 
-        GDALPamRasterBand *poBand = cpl::down_cast<GDALPamRasterBand *>(
-            GetRasterBand(nBand) );
+        GDALRasterBand *poBand = GetRasterBand(nBand);
 
         if( poBand == nullptr || !(poBand->GetMOFlags() & GMO_PAM_CLASS) )
             continue;
 
-        poBand->XMLInit( psBandTree, pszUnused );
+        GDALPamRasterBand *poPamBand = cpl::down_cast<GDALPamRasterBand *>(
+            GetRasterBand(nBand) );
+
+        poPamBand->XMLInit( psBandTree, pszUnused );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Preserve Array information.                                     */
+/* -------------------------------------------------------------------- */
+    for( CPLXMLNode* psIter = psTree->psChild;
+                                            psIter; psIter = psIter->psNext )
+    {
+        if( psIter->eType == CXT_Element &&
+            strcmp(psIter->pszValue, "Array") == 0 )
+        {
+            CPLXMLNode* psNextBackup = psIter->psNext;
+            psIter->psNext = nullptr;
+            psPam->m_apoOtherNodes.emplace_back(CPLXMLTreeCloser(CPLCloneXMLTree(psIter)));
+            psIter->psNext = psNextBackup;
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -679,7 +796,8 @@ CPLErr GDALPamDataset::TryLoadXML(char **papszSiblingFiles)
     int nLastErrNo = CPLGetLastErrorNo();
     CPLString osLastErrorMsg = CPLGetLastErrorMsg();
 
-    if (papszSiblingFiles != nullptr && IsPamFilenameAPotentialSiblingFile())
+    if( papszSiblingFiles != nullptr && IsPamFilenameAPotentialSiblingFile() &&
+        GDALCanReliablyUseSiblingFileList(psPam->pszPamFilename) )
     {
         const int iSibling =
             CSLFindString( papszSiblingFiles,
@@ -843,6 +961,15 @@ CPLErr GDALPamDataset::TrySaveXML()
         psTree = psOldTree;
     }
 
+
+/* -------------------------------------------------------------------- */
+/*      Preserve other information.                                     */
+/* -------------------------------------------------------------------- */
+    for( const auto& poOtherNode: psPam->m_apoOtherNodes )
+    {
+        CPLAddXMLChild(psTree, CPLCloneXMLTree(poOtherNode.get()));
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Try saving the auxiliary metadata.                               */
 /* -------------------------------------------------------------------- */
@@ -933,14 +1060,13 @@ CPLErr GDALPamDataset::CloneInfo( GDALDataset *poSrcDS, int nCloneFlags )
 /* -------------------------------------------------------------------- */
     if( nCloneFlags & GCIF_PROJECTION )
     {
-        const char *pszWKT = poSrcDS->GetProjectionRef();
+        const auto poSRS = poSrcDS->GetSpatialRef();
 
-        if( pszWKT != nullptr && strlen(pszWKT) > 0 )
+        if( poSRS != nullptr )
         {
             if( !bOnlyIfMissing
-                || GetProjectionRef() == nullptr
-                || strlen(GetProjectionRef()) == 0 )
-                SetProjection( pszWKT );
+                || GetSpatialRef() == nullptr )
+                SetSpatialRef( poSRS );
         }
     }
 
@@ -955,7 +1081,7 @@ CPLErr GDALPamDataset::CloneInfo( GDALDataset *poSrcDS, int nCloneFlags )
             {
                 SetGCPs( poSrcDS->GetGCPCount(),
                          poSrcDS->GetGCPs(),
-                         poSrcDS->GetGCPProjection() );
+                         poSrcDS->GetGCPSpatialRef() );
             }
         }
     }
@@ -965,21 +1091,16 @@ CPLErr GDALPamDataset::CloneInfo( GDALDataset *poSrcDS, int nCloneFlags )
 /* -------------------------------------------------------------------- */
     if( nCloneFlags & GCIF_METADATA )
     {
-        if( poSrcDS->GetMetadata() != nullptr )
+        for( const char* pszMDD: { "", "RPC", "json:ISIS3", "json:VICAR" } )
         {
-            if( !bOnlyIfMissing
-                || CSLCount(GetMetadata()) != CSLCount(poSrcDS->GetMetadata()) )
+            auto papszSrcMD = poSrcDS->GetMetadata(pszMDD);
+            if(papszSrcMD  != nullptr )
             {
-                SetMetadata( poSrcDS->GetMetadata() );
-            }
-        }
-        if( poSrcDS->GetMetadata("RPC") != nullptr )
-        {
-            if( !bOnlyIfMissing
-                || CSLCount(GetMetadata("RPC"))
-                   != CSLCount(poSrcDS->GetMetadata("RPC")) )
-            {
-                SetMetadata( poSrcDS->GetMetadata("RPC"), "RPC" );
+                if( !bOnlyIfMissing
+                    || CSLCount(GetMetadata(pszMDD)) != CSLCount(papszSrcMD) )
+                {
+                    SetMetadata( papszSrcMD, pszMDD );
+                }
             }
         }
     }
@@ -991,15 +1112,16 @@ CPLErr GDALPamDataset::CloneInfo( GDALDataset *poSrcDS, int nCloneFlags )
     {
         for( int iBand = 0; iBand < GetRasterCount(); iBand++ )
         {
-            GDALPamRasterBand *poBand = cpl::down_cast<GDALPamRasterBand *>(
-                GetRasterBand(iBand+1) );
+            GDALRasterBand *poBand = GetRasterBand(iBand+1);
 
             if( poBand == nullptr || !(poBand->GetMOFlags() & GMO_PAM_CLASS) )
                 continue;
 
             if( poSrcDS->GetRasterCount() >= iBand+1 )
-                poBand->CloneInfo( poSrcDS->GetRasterBand(iBand+1),
-                                   nCloneFlags );
+            {
+                cpl::down_cast<GDALPamRasterBand *>(poBand)->
+                    CloneInfo( poSrcDS->GetRasterBand(iBand+1), nCloneFlags );
+            }
             else
                 CPLDebug(
                     "GDALPamDataset",
@@ -1040,6 +1162,7 @@ char **GDALPamDataset::GetFileList()
     char **papszFileList = GDALDataset::GetFileList();
 
     if( psPam && !psPam->osPhysicalFilename.empty()
+        && GDALCanReliablyUseSiblingFileList(psPam->osPhysicalFilename.c_str())
         && CSLFindString( papszFileList, psPam->osPhysicalFilename ) == -1 )
     {
         papszFileList = CSLInsertString( papszFileList, 0,
@@ -1052,7 +1175,9 @@ char **GDALPamDataset::GetFileList()
         if (!bAddPamFile)
         {
             VSIStatBufL sStatBuf;
-            if (oOvManager.GetSiblingFiles() != nullptr && IsPamFilenameAPotentialSiblingFile())
+            if (oOvManager.GetSiblingFiles() != nullptr &&
+                IsPamFilenameAPotentialSiblingFile() &&
+                GDALCanReliablyUseSiblingFileList(psPam->pszPamFilename) )
             {
                 bAddPamFile = CSLFindString(oOvManager.GetSiblingFiles(),
                                   CPLGetFilename(psPam->pszPamFilename)) >= 0;
@@ -1070,6 +1195,7 @@ char **GDALPamDataset::GetFileList()
     }
 
     if( psPam && !psPam->osAuxFilename.empty() &&
+        GDALCanReliablyUseSiblingFileList(psPam->osAuxFilename.c_str()) &&
         CSLFindString( papszFileList, psPam->osAuxFilename ) == -1 )
     {
         papszFileList = CSLAddString( papszFileList, psPam->osAuxFilename );
@@ -1121,32 +1247,33 @@ CPLErr GDALPamDataset::IBuildOverviews( const char *pszResampling,
 //! @endcond
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                           GetSpatialRef()                            */
 /************************************************************************/
 
-const char *GDALPamDataset::GetProjectionRef()
+const OGRSpatialReference *GDALPamDataset::GetSpatialRef() const
 
 {
-    if( psPam && psPam->pszProjection )
-        return psPam->pszProjection;
+    if( psPam && psPam->poSRS )
+        return psPam->poSRS;
 
-    return GDALDataset::GetProjectionRef();
+    return GDALDataset::GetSpatialRef();
 }
 
 /************************************************************************/
-/*                           SetProjection()                            */
+/*                           SetSpatialRef()                            */
 /************************************************************************/
 
-CPLErr GDALPamDataset::SetProjection( const char *pszProjectionIn )
+CPLErr GDALPamDataset::SetSpatialRef( const OGRSpatialReference* poSRS )
 
 {
     PamInitialize();
 
     if( psPam == nullptr )
-        return GDALDataset::SetProjection( pszProjectionIn );
+        return GDALDataset::SetSpatialRef( poSRS );
 
-    CPLFree( psPam->pszProjection );
-    psPam->pszProjection = CPLStrdup( pszProjectionIn );
+    if( psPam->poSRS )
+        psPam->poSRS->Release();
+    psPam->poSRS = poSRS ? poSRS->Clone() : nullptr;
     MarkPamDirty();
 
     return CE_None;
@@ -1202,16 +1329,16 @@ int GDALPamDataset::GetGCPCount()
 }
 
 /************************************************************************/
-/*                          GetGCPProjection()                          */
+/*                          GetGCPSpatialRef()                          */
 /************************************************************************/
 
-const char *GDALPamDataset::GetGCPProjection()
+const OGRSpatialReference *GDALPamDataset::GetGCPSpatialRef() const
 
 {
-    if( psPam && psPam->pszGCPProjection != nullptr )
-        return psPam->pszGCPProjection;
+    if( psPam && psPam->poGCP_SRS != nullptr )
+        return psPam->poGCP_SRS;
 
-    return GDALDataset::GetGCPProjection();
+    return GDALDataset::GetGCPSpatialRef();
 }
 
 /************************************************************************/
@@ -1232,21 +1359,22 @@ const GDAL_GCP *GDALPamDataset::GetGCPs()
 /************************************************************************/
 
 CPLErr GDALPamDataset::SetGCPs( int nGCPCount, const GDAL_GCP *pasGCPList,
-                                const char *pszGCPProjection )
+                                const OGRSpatialReference* poGCP_SRS )
 
 {
     PamInitialize();
 
     if( psPam )
     {
-        CPLFree( psPam->pszGCPProjection );
+        if( psPam->poGCP_SRS )
+            psPam->poGCP_SRS->Release();
         if( psPam->nGCPCount > 0 )
         {
             GDALDeinitGCPs( psPam->nGCPCount, psPam->pasGCPList );
             CPLFree( psPam->pasGCPList );
         }
 
-        psPam->pszGCPProjection = CPLStrdup(pszGCPProjection);
+        psPam->poGCP_SRS = poGCP_SRS ? poGCP_SRS->Clone() : nullptr;
         psPam->nGCPCount = nGCPCount;
         psPam->pasGCPList = GDALDuplicateGCPs( nGCPCount, pasGCPList );
 
@@ -1255,7 +1383,7 @@ CPLErr GDALPamDataset::SetGCPs( int nGCPCount, const GDAL_GCP *pasGCPList,
         return CE_None;
     }
 
-    return GDALDataset::SetGCPs( nGCPCount, pasGCPList, pszGCPProjection );
+    return GDALDataset::SetGCPs( nGCPCount, pasGCPList, poGCP_SRS );
 }
 
 /************************************************************************/
@@ -1399,7 +1527,7 @@ CPLErr GDALPamDataset::TryLoadAux(char **papszSiblingFiles)
     if( strlen(pszPhysicalFile) == 0 )
         return CE_None;
 
-    if( papszSiblingFiles )
+    if( papszSiblingFiles && GDALCanReliablyUseSiblingFileList(pszPhysicalFile) )
     {
         CPLString osAuxFilename = CPLResetExtension( pszPhysicalFile, "aux");
         int iSibling = CSLFindString( papszSiblingFiles,
@@ -1536,3 +1664,108 @@ CPLErr GDALPamDataset::TryLoadAux(char **papszSiblingFiles)
     return CE_Failure;
 }
 //! @endcond
+
+/************************************************************************/
+/*                        _GetProjectionRef()                           */
+/************************************************************************/
+
+//! @cond Doxygen_Suppress
+const char *GDALPamDataset::_GetProjectionRef()
+{
+    return GetProjectionRefFromSpatialRef(GDALPamDataset::GetSpatialRef());
+}
+
+/************************************************************************/
+/*                          _SetProjection()                            */
+/************************************************************************/
+
+CPLErr GDALPamDataset::_SetProjection( const char *pszProjection )
+{
+    if( pszProjection && pszProjection[0] != '\0' )
+    {
+        OGRSpatialReference oSRS;
+        oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+        if( oSRS.importFromWkt(pszProjection) != OGRERR_NONE )
+        {
+            return CE_Failure;
+        }
+        return GDALPamDataset::SetSpatialRef(&oSRS);
+    }
+    else
+    {
+        return GDALPamDataset::SetSpatialRef(nullptr);
+    }
+}
+
+/************************************************************************/
+/*                        _GetGCPProjection()                           */
+/************************************************************************/
+
+const char *GDALPamDataset::_GetGCPProjection()
+{
+    return GetGCPProjectionFromSpatialRef(GDALPamDataset::GetGCPSpatialRef());
+}
+
+/************************************************************************/
+/*                            _SetGCPs()                                */
+/************************************************************************/
+
+CPLErr GDALPamDataset::_SetGCPs( int nGCPCount,
+                             const GDAL_GCP *pasGCPList,
+                             const char *pszGCPProjection )
+
+{
+    if( pszGCPProjection && pszGCPProjection[0] != '\0' )
+    {
+        OGRSpatialReference oSRS;
+        oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+        if( oSRS.importFromWkt(pszGCPProjection) != OGRERR_NONE )
+        {
+            return CE_Failure;
+        }
+        return GDALPamDataset::SetGCPs(nGCPCount, pasGCPList, &oSRS);
+    }
+    else
+    {
+        return GDALPamDataset::SetGCPs(nGCPCount, pasGCPList,
+                       static_cast<const OGRSpatialReference*>(nullptr));
+    }
+}
+//! @endcond
+
+/************************************************************************/
+/*                          ClearStatistics()                           */
+/************************************************************************/
+
+void GDALPamDataset::ClearStatistics()
+{
+    PamInitialize();
+    if( !psPam )
+        return;
+    for( int i = 1; i <= nBands; ++i )
+    {
+        bool bChanged = false;
+        GDALRasterBand* poBand = GetRasterBand(i);
+        char** papszOldMD = poBand->GetMetadata();
+        char** papszNewMD = nullptr;
+        for( char** papszIter = papszOldMD; papszIter && papszIter[0]; ++papszIter )
+        {
+            if( STARTS_WITH_CI(*papszIter, "STATISTICS_") )
+            {
+                MarkPamDirty();
+                bChanged = true;
+            }
+            else
+            {
+                papszNewMD = CSLAddString(papszNewMD, *papszIter);
+            }
+        }
+        if( bChanged )
+        {
+            poBand->SetMetadata(papszNewMD);
+        }
+        CSLDestroy(papszNewMD);
+    }
+
+    GDALDataset::ClearStatistics();
+}
