@@ -34,7 +34,7 @@
 #include <limits>
 #include <set>
 
-CPL_CVSID("$Id: gnmgraph.cpp 7e07230bbff24eb333608de4dbd460b7312839d0 2017-12-11 19:08:47Z Even Rouault $")
+CPL_CVSID("$Id: gnmgraph.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 //! @cond Doxygen_Suppress
 GNMGraph::GNMGraph() {}
@@ -47,7 +47,7 @@ void GNMGraph::AddVertex(GNMGFID nFID)
         return;
 
     GNMStdVertex stVertex;
-    stVertex.bIsBloked = false;
+    stVertex.bIsBlocked = false;
     m_mstVertices[nFID] = stVertex;
 }
 
@@ -92,7 +92,7 @@ void GNMGraph::AddEdge(GNMGFID nConFID, GNMGFID nSrcFID, GNMGFID nTgtFID,
     stEdge.bIsBidir = bIsBidir;
     stEdge.dfDirCost = dfCost;
     stEdge.dfInvCost = dfInvCost;
-    stEdge.bIsBloked = false;
+    stEdge.bIsBlocked = false;
 
     m_mstEdges[nConFID] = stEdge;
 
@@ -138,7 +138,7 @@ void GNMGraph::ChangeBlockState(GNMGFID nFID, bool bBlock)
     std::map<GNMGFID, GNMStdVertex>::iterator itv = m_mstVertices.find(nFID);
     if(itv != m_mstVertices.end())
     {
-        itv->second.bIsBloked = bBlock;
+        itv->second.bIsBlocked = bBlock;
         return;
     }
 
@@ -146,7 +146,7 @@ void GNMGraph::ChangeBlockState(GNMGFID nFID, bool bBlock)
     std::map<GNMGFID, GNMStdEdge>::iterator ite = m_mstEdges.find(nFID);
     if (ite != m_mstEdges.end())
     {
-        ite->second.bIsBloked = bBlock;
+        ite->second.bIsBlocked = bBlock;
     }
 }
 
@@ -154,7 +154,7 @@ bool GNMGraph::CheckVertexBlocked(GNMGFID nFID) const
 {
     std::map<GNMGFID, GNMStdVertex>::const_iterator it = m_mstVertices.find(nFID);
     if (it != m_mstVertices.end())
-        return it->second.bIsBloked;
+        return it->second.bIsBlocked;
     return false;
 }
 
@@ -163,13 +163,13 @@ void GNMGraph::ChangeAllBlockState(bool bBlock)
     for(std::map<GNMGFID, GNMStdVertex>::iterator itv = m_mstVertices.begin();
         itv != m_mstVertices.end(); ++itv)
     {
-        itv->second.bIsBloked = bBlock;
+        itv->second.bIsBlocked = bBlock;
     }
 
     for(std::map<GNMGFID, GNMStdEdge>::iterator ite = m_mstEdges.begin();
         ite != m_mstEdges.end(); ++ite)
     {
-        ite->second.bIsBloked = bBlock;
+        ite->second.bIsBlocked = bBlock;
     }
 }
 
@@ -219,7 +219,7 @@ GNMPATH GNMGraph::DijkstraShortestPath( GNMGFID nStartFID, GNMGFID nEndFID,
             // final array.
             aoShortestPath.push_back(std::make_pair(nNextVertexId, it->second));
 
-            // An edge has only two vertexes, so we get the opposite one to the
+            // An edge has only two vertices, so we get the opposite one to the
             // current vertex in order to continue search backwards.
             nNextVertexId = GetOppositVertex(it->second, it->first);
         }
@@ -448,7 +448,7 @@ void GNMGraph::DijkstraShortestPathTree(GNMGFID nFID,
                                    const std::map<GNMGFID, GNMStdEdge> &mstEdges,
                                          std::map<GNMGFID, GNMGFID> &mnPathTree)
 {
-    // Initialize all vertexes in graph with infinity mark.
+    // Initialize all vertices in graph with infinity mark.
     double dfInfinity = std::numeric_limits<double>::infinity();
 
     std::map<GNMGFID, double> mMarks;
@@ -461,35 +461,35 @@ void GNMGraph::DijkstraShortestPathTree(GNMGFID nFID,
     mMarks[nFID] = 0.0;
     mnPathTree[nFID] = -1;
 
-    // Initialize all vertexes as unseen (there are no seen vertexes).
+    // Initialize all vertices as unseen (there are no seen vertices).
     std::set<GNMGFID> snSeen;
 
     // We use multimap to maintain the ascending order of costs and because
-    // there can be different vertexes with the equal cost.
+    // there can be different vertices with the equal cost.
     std::multimap<double,GNMGFID> to_see;
     std::multimap<double,GNMGFID>::iterator it;
     to_see.insert(std::pair<double,GNMGFID>(0.0, nFID));
     LPGNMCONSTVECTOR panOutcomeEdgeId;
 
     size_t i;
-    GNMGFID nCurrenVertId, nCurrentEdgeId, nTargetVertId;
+    GNMGFID nCurrentVertId, nCurrentEdgeId, nTargetVertId;
     double dfCurrentEdgeCost, dfCurrentVertMark, dfNewVertexMark;
     std::map<GNMGFID, GNMStdEdge>::const_iterator ite;
 
-    // Continue iterations while there are some vertexes to see.
+    // Continue iterations while there are some vertices to see.
     while (!to_see.empty())
     {
-        // We must see vertexes with minimal costs at first.
+        // We must see vertices with minimal costs at first.
         // In multimap the first cost is the minimal.
         it = to_see.begin();
 
-        nCurrenVertId = it->second;
+        nCurrentVertId = it->second;
         dfCurrentVertMark = it->first;
         snSeen.insert(it->second);
         to_see.erase(it);
 
         // For all neighbours for the current vertex.
-        panOutcomeEdgeId = GetOutEdges(nCurrenVertId);
+        panOutcomeEdgeId = GetOutEdges(nCurrentVertId);
         if(nullptr == panOutcomeEdgeId)
             continue;
 
@@ -498,7 +498,7 @@ void GNMGraph::DijkstraShortestPathTree(GNMGFID nFID,
             nCurrentEdgeId = panOutcomeEdgeId->operator[](i);
 
             ite = mstEdges.find(nCurrentEdgeId);
-            if(ite == mstEdges.end() || ite->second.bIsBloked)
+            if(ite == mstEdges.end() || ite->second.bIsBlocked)
                 continue;
 
             // We go in any edge from source to target so we take only
@@ -507,7 +507,7 @@ void GNMGraph::DijkstraShortestPathTree(GNMGFID nFID,
 
             // While we see outcome edges of current vertex id we definitely
             // know that target vertex id will be target for current edge id.
-            nTargetVertId = GetOppositVertex(nCurrentEdgeId, nCurrenVertId);
+            nTargetVertId = GetOppositVertex(nCurrentEdgeId, nCurrentVertId);
 
             // Calculate a new mark assuming the full path cost (mark of the
             // current vertex) to this vertex.
@@ -562,12 +562,12 @@ void GNMGraph::TraceTargets(std::queue<GNMGFID> &vertexQueue,
     GNMCONSTVECTOR::const_iterator it;
     std::queue<GNMGFID> neighbours_queue;
 
-    // See all given vertexes except thouse that have been already seen.
+    // See all given vertices except thouse that have been already seen.
     while (!vertexQueue.empty())
     {
         GNMGFID nCurVertID = vertexQueue.front();
 
-        // There may be duplicate unmarked vertexes in a current queue. Check it.
+        // There may be duplicate unmarked vertices in a current queue. Check it.
         if (markedVertIds.find(nCurVertID) == markedVertIds.end())
         {
             markedVertIds.insert(nCurVertID);
@@ -582,7 +582,7 @@ void GNMGraph::TraceTargets(std::queue<GNMGFID> &vertexQueue,
                 {
                     GNMGFID nCurEdgeID = *it;
 
-                    // ISSUE: think about to return a sequence of vertexes and edges
+                    // ISSUE: think about to return a sequence of vertices and edges
                     // (which is more universal), as now we are going to return only
                     // sequence of edges.
                     connectedIds.push_back( std::make_pair(nCurVertID, nCurEdgeID) );
@@ -604,7 +604,7 @@ void GNMGraph::TraceTargets(std::queue<GNMGFID> &vertexQueue,
                     */
                     nTargetVertID = GetOppositVertex(nCurEdgeID, nCurVertID);
 
-                    // Avoid marked or blocked vertexes.
+                    // Avoid marked or blocked vertices.
                     if ((markedVertIds.find(nTargetVertID) == markedVertIds.end())
                     && (!CheckVertexBlocked(nTargetVertID)))
                         neighbours_queue.push(nTargetVertID);
