@@ -2,10 +2,10 @@
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of OGRTopoJSONReader class
- * Author:   Even Rouault, even dot rouault at mines dash paris dot org
+ * Author:   Even Rouault, even dot rouault at spatialys.com
  *
  ******************************************************************************
- * Copyright (c) 2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,9 +30,9 @@
 #include "ogrgeojsonutils.h"
 #include "ogr_geojson.h"
 #include <json.h>  // JSON-C
-#include <ogr_api.h>
+#include "ogr_api.h"
 
-CPL_CVSID("$Id: ogrtopojsonreader.cpp c3b17f3db1f346b23bbab3000bd3b5e4225b7512 2019-01-03 10:26:12 -0500 alanstewart-terragotech $")
+CPL_CVSID("$Id: ogrtopojsonreader.cpp f28f5e8137ffeb479da480dc99e4dedbeec895ab 2021-09-26 20:02:01 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                          OGRTopoJSONReader()                         */
@@ -58,10 +58,20 @@ OGRTopoJSONReader::~OGRTopoJSONReader()
 /*                           Parse()                                    */
 /************************************************************************/
 
-OGRErr OGRTopoJSONReader::Parse( const char* pszText )
+OGRErr OGRTopoJSONReader::Parse( const char* pszText, bool bLooseIdentification )
 {
     json_object *jsobj = nullptr;
-    if( nullptr != pszText && !OGRJSonParse(pszText, &jsobj, true) )
+    if( bLooseIdentification )
+    {
+        CPLPushErrorHandler(CPLQuietErrorHandler);
+    }
+    const bool bOK = nullptr != pszText && OGRJSonParse(pszText, &jsobj, true);
+    if( bLooseIdentification )
+    {
+        CPLPopErrorHandler();
+        CPLErrorReset();
+    }
+    if( !bOK )
     {
         return OGRERR_CORRUPT_DATA;
     }
@@ -416,7 +426,10 @@ static void ParseObject( const char* pszId,
 /*                        EstablishLayerDefn()                          */
 /************************************************************************/
 
-static void EstablishLayerDefn( OGRFeatureDefn* poDefn,
+static void EstablishLayerDefn( std::vector<int>& anCurFieldIndices,
+                                std::map<std::string, int>& oMapFieldNameToIdx,
+                                std::vector<std::unique_ptr<OGRFieldDefn>>& apoFieldDefn,
+                                gdal::DirectedAcyclicGraph<int, std::string>& dag,
                                 json_object* poObj,
                                 std::set<int>& aoSetUndeterminedTypeFields )
 {
@@ -429,11 +442,25 @@ static void EstablishLayerDefn( OGRFeatureDefn* poDefn,
         it.val = nullptr;
         it.entry = nullptr;
 
+        int nPrevFieldIdx = -1;
         json_object_object_foreachC( poObjProps, it )
         {
-            OGRGeoJSONReaderAddOrUpdateField(poDefn, it.key, it.val,
-                                             false, 0, false,
+            anCurFieldIndices.clear();
+            OGRGeoJSONReaderAddOrUpdateField(anCurFieldIndices,
+                                             oMapFieldNameToIdx,
+                                             apoFieldDefn,
+                                             it.key, it.val,
+                                             false, 0, false, false,
                                              aoSetUndeterminedTypeFields);
+            for( int idx: anCurFieldIndices )
+            {
+                dag.addNode(idx, apoFieldDefn[idx]->GetNameRef());
+                if( nPrevFieldIdx != -1 )
+                {
+                    dag.addEdge(nPrevFieldIdx, idx);
+                }
+                nPrevFieldIdx = idx;
+            }
         }
     }
 }
@@ -490,6 +517,11 @@ static bool ParseObjectMain( const char* pszId, json_object* poObj,
                     const auto nGeometries =
                         json_object_array_length(poGeometries);
                     // First pass to establish schema.
+                    std::vector<int> anCurFieldIndices;
+                    std::map<std::string, int> oMapFieldNameToIdx;
+                    std::vector<std::unique_ptr<OGRFieldDefn>> apoFieldDefn;
+                    gdal::DirectedAcyclicGraph<int, std::string> dag;
+
                     for( auto i = decltype(nGeometries){0}; i < nGeometries;
                          i++ )
                     {
@@ -498,11 +530,22 @@ static bool ParseObjectMain( const char* pszId, json_object* poObj,
                         if( poGeom != nullptr &&
                             json_type_object == json_object_get_type( poGeom ) )
                         {
-                            EstablishLayerDefn(poDefn, poGeom,
+                            EstablishLayerDefn(anCurFieldIndices,
+                                               oMapFieldNameToIdx,
+                                               apoFieldDefn,
+                                               dag,
+                                               poGeom,
                                                aoSetUndeterminedTypeFields);
                         }
                     }
 
+                    const auto sortedFields = dag.getTopologicalOrdering();
+                    CPLAssert( sortedFields.size() == apoFieldDefn.size() );
+                    for( int idx: sortedFields )
+                    {
+                        poDefn->AddFieldDefn(apoFieldDefn[idx].get());
+                    }
+
                     // Second pass to build objects.
                     for( auto i = decltype(nGeometries){0}; i < nGeometries;
                          i++ )
@@ -540,9 +583,26 @@ static bool ParseObjectMain( const char* pszId, json_object* poObj,
                                 GetLayerDefn()->AddFieldDefn( &fldDefn );
                         }
                     }
-                    OGRFeatureDefn* poDefn = (*ppoMainLayer)->GetLayerDefn();
-                    EstablishLayerDefn(poDefn, poObj,
+                    std::vector<int> anCurFieldIndices;
+                    std::map<std::string, int> oMapFieldNameToIdx;
+                    std::vector<std::unique_ptr<OGRFieldDefn>> apoFieldDefn;
+                    gdal::DirectedAcyclicGraph<int, std::string> dag;
+
+                    EstablishLayerDefn(anCurFieldIndices,
+                                       oMapFieldNameToIdx,
+                                       apoFieldDefn,
+                                       dag,
+                                       poObj,
                                        aoSetUndeterminedTypeFields);
+                    OGRFeatureDefn* poDefn = (*ppoMainLayer)->GetLayerDefn();
+
+                    const auto sortedFields = dag.getTopologicalOrdering();
+                    CPLAssert( sortedFields.size() == apoFieldDefn.size() );
+                    for( int idx: sortedFields )
+                    {
+                        poDefn->AddFieldDefn(apoFieldDefn[idx].get());
+                    }
+
                     bNeedSecondPass = true;
                 }
                 else
