@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_pg.h 3798cbe48457b7127606931896549f26507469db 2021-04-09 15:04:16 +0200 Even Rouault $
+ * $Id: ogr_pg.h 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for OGR/PostgreSQL driver.
@@ -154,8 +154,9 @@ class OGRPGFeatureDefn CPL_NON_FINAL: public OGRFeatureDefn
 
         virtual void UnsetLayer()
         {
+            const int nGeomFieldCount = GetGeomFieldCount();
             for(int i=0;i<nGeomFieldCount;i++)
-                cpl::down_cast<OGRPGGeomFieldDefn*>(papoGeomFieldDefn[i])->UnsetLayer();
+                cpl::down_cast<OGRPGGeomFieldDefn*>(apoGeomFieldDefn[i].get())->UnsetLayer();
         }
 
         OGRPGGeomFieldDefn *GetGeomFieldDefn( int i ) override
@@ -331,8 +332,8 @@ class OGRPGTableLayer final: public OGRPGLayer
 
     virtual CPLString   GetFromClauseForGetExtent() override { return pszSqlTableName; }
 
-    OGRErr              RunAddGeometryColumn( OGRPGGeomFieldDefn *poGeomField );
-    OGRErr              RunCreateSpatialIndex( OGRPGGeomFieldDefn *poGeomField );
+    OGRErr              RunAddGeometryColumn( const OGRPGGeomFieldDefn *poGeomField );
+    OGRErr              RunCreateSpatialIndex( const OGRPGGeomFieldDefn *poGeomField );
 
     void                UpdateSequenceIfNeeded();
 
@@ -565,7 +566,7 @@ class OGRPGDataSource final: public OGRDataSource
     OGRLayer            *GetLayer( int ) override;
     OGRLayer            *GetLayerByName(const char * pszName) override;
 
-    virtual void        FlushCache() override;
+    virtual void        FlushCache(bool bAtClosing) override;
 
     virtual OGRLayer    *ICreateLayer( const char *,
                                       OGRSpatialReference * = nullptr,
